@@ -1,17 +1,6 @@
-//! Elliptic Curve Digital Signature Algorithm (ECDSA) as specified in
-//! [FIPS 186-4] (Digital Signature Standard)
-//!
-//! ## About
-//!
-//! This crate provides generic ECDSA support which can be used in the
-//! following ways:
-//!
-//! - Generic implementation of ECDSA usable with the following crates:
-//!   - [`k256`] (secp256k1)
-//!   - [`p256`] (NIST P-256)
-//! - ECDSA signature types alone which can be used to provide interoperability
-//!   between other crates that provide an ECDSA implementation:
-//!   - [`p384`] (NIST P-384)
+#![doc = include_str!("../README.md")]
+
+//! ## Interop
 //!
 //! Any crates which provide an implementation of ECDSA for a particular
 //! elliptic curve can leverage the types from this crate, along with the
@@ -26,14 +15,6 @@
 //! wrap the ECDSA implementations from [*ring*] in a generic, interoperable
 //! API.
 //!
-//! ## Minimum Supported Rust Version
-//!
-//! Rust **1.51** or higher.
-//!
-//! Minimum supported Rust version may be changed in the future, but it will be
-//! accompanied with a minor version bump.
-//!
-//! [FIPS 186-4]: https://csrc.nist.gov/publications/detail/fips/186/4/final
 //! [`k256`]: https://docs.rs/k256
 //! [`p256`]: https://docs.rs/p256
 //! [`p256::ecdsa::Signature`]: https://docs.rs/p256/latest/p256/ecdsa/type.Signature.html
@@ -49,12 +30,14 @@
 #![doc(
     html_logo_url = "https://raw.githubusercontent.com/RustCrypto/media/8f1a9894/logo.svg",
     html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/media/8f1a9894/logo.svg",
-    html_root_url = "https://docs.rs/ecdsa/0.12.4"
+    html_root_url = "https://docs.rs/ecdsa/0.13.0"
 )]
 
 #[cfg(feature = "alloc")]
 extern crate alloc;
 
+mod recovery;
+
 #[cfg(feature = "der")]
 #[cfg_attr(docsrs, doc(cfg(feature = "der")))]
 pub mod der;
@@ -67,43 +50,54 @@ pub mod dev;
 #[cfg_attr(docsrs, doc(cfg(feature = "hazmat")))]
 pub mod hazmat;
 
-#[cfg(feature = "sign")]
-#[cfg_attr(docsrs, doc(cfg(feature = "sign")))]
-pub mod rfc6979;
-
 #[cfg(feature = "sign")]
 mod sign;
 
 #[cfg(feature = "verify")]
 mod verify;
 
+pub use crate::recovery::RecoveryId;
+
 // Re-export the `elliptic-curve` crate (and select types)
-pub use elliptic_curve::{self, sec1::EncodedPoint, weierstrass::Curve};
+pub use elliptic_curve::{self, sec1::EncodedPoint, PrimeCurve};
 
 // Re-export the `signature` crate (and select types)
 pub use signature::{self, Error, Result};
 
+#[cfg(feature = "rfc6979")]
+#[cfg_attr(docsrs, doc(cfg(feature = "rfc6979")))]
+pub use rfc6979;
+
 #[cfg(feature = "sign")]
 #[cfg_attr(docsrs, doc(cfg(feature = "sign")))]
-pub use sign::SigningKey;
+pub use crate::sign::SigningKey;
 
 #[cfg(feature = "verify")]
 #[cfg_attr(docsrs, doc(cfg(feature = "verify")))]
-pub use verify::VerifyingKey;
+pub use crate::verify::VerifyingKey;
 
 use core::{
-    convert::TryFrom,
     fmt::{self, Debug},
     ops::Add,
 };
 use elliptic_curve::{
     bigint::Encoding as _,
-    generic_array::{sequence::Concat, typenum::Unsigned, ArrayLength, GenericArray},
-    FieldBytes, FieldSize, ScalarBytes,
+    generic_array::{sequence::Concat, ArrayLength, GenericArray},
+    FieldBytes, FieldSize, ScalarCore,
 };
 
 #[cfg(feature = "arithmetic")]
-use elliptic_curve::{group::ff::PrimeField, NonZeroScalar, ProjectiveArithmetic, Scalar};
+use {
+    core::str,
+    elliptic_curve::{ff::PrimeField, IsHigh, NonZeroScalar, ScalarArithmetic},
+};
+
+#[cfg(feature = "serde")]
+use elliptic_curve::serde::{ser, Serialize};
+
+// TODO(tarcieri): support deserialization with the `arithmetic` feature disabled
+#[cfg(all(feature = "arithmetic", feature = "serde"))]
+use elliptic_curve::serde::{de, Deserialize};
 
 /// Size of a fixed sized signature for the given elliptic curve.
 pub type SignatureSize<C> = <FieldSize<C> as Add>::Output;
@@ -125,7 +119,7 @@ pub type SignatureBytes<C> = GenericArray<u8, SignatureSize<C>>;
 /// ASN.1 DER-encoded signatures also supported via the
 /// [`Signature::from_der`] and [`Signature::to_der`] methods.
 #[derive(Clone, Eq, PartialEq)]
-pub struct Signature<C: Curve>
+pub struct Signature<C: PrimeCurve>
 where
     SignatureSize<C>: ArrayLength<u8>,
 {
@@ -134,15 +128,9 @@ where
 
 impl<C> Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     SignatureSize<C>: ArrayLength<u8>,
 {
-    /// Create a [`Signature`] from the serialized `r` and `s` scalar values
-    /// which comprise the signature.
-    pub fn from_scalars(r: impl Into<FieldBytes<C>>, s: impl Into<FieldBytes<C>>) -> Result<Self> {
-        Self::try_from(r.into().concat(s.into()).as_slice())
-    }
-
     /// Parse a signature from ASN.1 DER
     #[cfg(feature = "der")]
     #[cfg_attr(docsrs, doc(cfg(feature = "der")))]
@@ -154,6 +142,22 @@ where
         der::Signature::<C>::try_from(bytes).and_then(Self::try_from)
     }
 
+    /// Create a [`Signature`] from the serialized `r` and `s` scalar values
+    /// which comprise the signature.
+    pub fn from_scalars(r: impl Into<FieldBytes<C>>, s: impl Into<FieldBytes<C>>) -> Result<Self> {
+        Self::try_from(r.into().concat(s.into()).as_slice())
+    }
+
+    /// Split the signature into its `r` and `s` components, represented as bytes.
+    pub fn split_bytes(&self) -> (FieldBytes<C>, FieldBytes<C>) {
+        let (r_bytes, s_bytes) = self.bytes.split_at(C::UInt::BYTE_SIZE);
+
+        (
+            GenericArray::clone_from_slice(r_bytes),
+            GenericArray::clone_from_slice(s_bytes),
+        )
+    }
+
     /// Serialize this signature as ASN.1 DER
     #[cfg(feature = "der")]
     #[cfg_attr(docsrs, doc(cfg(feature = "der")))]
@@ -171,49 +175,47 @@ where
 #[cfg_attr(docsrs, doc(cfg(feature = "arithmetic")))]
 impl<C> Signature<C>
 where
-    C: Curve + ProjectiveArithmetic,
+    C: PrimeCurve + ScalarArithmetic,
     SignatureSize<C>: ArrayLength<u8>,
 {
     /// Get the `r` component of this signature
     pub fn r(&self) -> NonZeroScalar<C> {
-        let r_bytes = GenericArray::clone_from_slice(&self.bytes[..C::UInt::BYTE_SIZE]);
-        NonZeroScalar::from_repr(r_bytes)
-            .unwrap_or_else(|| unreachable!("r-component ensured valid in constructor"))
+        NonZeroScalar::try_from(self.split_bytes().0.as_slice())
+            .expect("r-component ensured valid in constructor")
     }
 
     /// Get the `s` component of this signature
     pub fn s(&self) -> NonZeroScalar<C> {
-        let s_bytes = GenericArray::clone_from_slice(&self.bytes[C::UInt::BYTE_SIZE..]);
-        NonZeroScalar::from_repr(s_bytes)
-            .unwrap_or_else(|| unreachable!("r-component ensured valid in constructor"))
+        NonZeroScalar::try_from(self.split_bytes().1.as_slice())
+            .expect("s-component ensured valid in constructor")
+    }
+
+    /// Split the signature into its `r` and `s` scalars.
+    pub fn split_scalars(&self) -> (NonZeroScalar<C>, NonZeroScalar<C>) {
+        (self.r(), self.s())
     }
 
     /// Normalize signature into "low S" form as described in
     /// [BIP 0062: Dealing with Malleability][1].
     ///
     /// [1]: https://github.com/bitcoin/bips/blob/master/bip-0062.mediawiki
-    pub fn normalize_s(&mut self) -> Result<bool>
-    where
-        Scalar<C>: NormalizeLow,
-    {
-        let s_bytes = GenericArray::from_mut_slice(&mut self.bytes[C::UInt::BYTE_SIZE..]);
-        Scalar::<C>::from_repr(s_bytes.clone())
-            .map(|s| {
-                let (s_low, was_high) = s.normalize_low();
+    pub fn normalize_s(&self) -> Option<Self> {
+        let s = self.s();
 
-                if was_high {
-                    s_bytes.copy_from_slice(&s_low.to_repr());
-                }
-
-                was_high
-            })
-            .ok_or_else(Error::new)
+        if s.is_high().into() {
+            let neg_s = -s;
+            let mut result = self.clone();
+            result.bytes[C::UInt::BYTE_SIZE..].copy_from_slice(&neg_s.to_repr());
+            Some(result)
+        } else {
+            None
+        }
     }
 }
 
 impl<C> signature::Signature for Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     SignatureSize<C>: ArrayLength<u8>,
 {
     fn from_bytes(bytes: &[u8]) -> Result<Self> {
@@ -223,7 +225,7 @@ where
 
 impl<C> AsRef<[u8]> for Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     SignatureSize<C>: ArrayLength<u8>,
 {
     fn as_ref(&self) -> &[u8] {
@@ -233,7 +235,7 @@ where
 
 impl<C> Copy for Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     SignatureSize<C>: ArrayLength<u8>,
     <SignatureSize<C> as ArrayLength<u8>>::ArrayType: Copy,
 {
@@ -241,7 +243,7 @@ where
 
 impl<C> Debug for Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     SignatureSize<C>: ArrayLength<u8>,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
@@ -256,25 +258,20 @@ where
 
 impl<C> TryFrom<&[u8]> for Signature<C>
 where
-    C: Curve,
+    C: PrimeCurve,
     SignatureSize<C>: ArrayLength<u8>,
 {
     type Error = Error;
 
     fn try_from(bytes: &[u8]) -> Result<Self> {
-        if bytes.len() != <SignatureSize<C>>::to_usize() {
+        if bytes.len() != C::UInt::BYTE_SIZE * 2 {
             return Err(Error::new());
         }
 
-        for scalar in bytes.chunks_exact(C::UInt::BYTE_SIZE) {
-            if scalar.iter().all(|&byte| byte == 0) {
-                return Err(Error::new());
-            }
+        for scalar_bytes in bytes.chunks_exact(C::UInt::BYTE_SIZE) {
+            let scalar = ScalarCore::<C>::from_be_slice(scalar_bytes).map_err(|_| Error::new())?;
 
-            if ScalarBytes::<C>::new(GenericArray::clone_from_slice(scalar))
-                .is_none()
-                .into()
-            {
+            if scalar.is_zero().into() {
                 return Err(Error::new());
             }
         }
@@ -285,16 +282,138 @@ where
     }
 }
 
-/// Normalize a scalar (i.e. ECDSA S) to the lower half the field, as described
-/// in [BIP 0062: Dealing with Malleability][1].
-///
-/// [1]: https://github.com/bitcoin/bips/blob/master/bip-0062.mediawiki
-pub trait NormalizeLow: Sized {
-    /// Normalize scalar to the lower half of the field (i.e. negate it if it's
-    /// larger than half the curve's order).
-    /// Returns a tuple with the new scalar and a boolean indicating whether the given scalar
-    /// was in the higher half.
-    ///
-    /// May be implemented to work in variable time.
-    fn normalize_low(&self) -> (Self, bool);
+impl<C> fmt::Display for Signature<C>
+where
+    C: PrimeCurve,
+    SignatureSize<C>: ArrayLength<u8>,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{:X}", self)
+    }
+}
+
+impl<C> fmt::LowerHex for Signature<C>
+where
+    C: PrimeCurve,
+    SignatureSize<C>: ArrayLength<u8>,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        for byte in &self.bytes {
+            write!(f, "{:02x}", byte)?;
+        }
+        Ok(())
+    }
+}
+
+impl<C> fmt::UpperHex for Signature<C>
+where
+    C: PrimeCurve,
+    SignatureSize<C>: ArrayLength<u8>,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        for byte in &self.bytes {
+            write!(f, "{:02X}", byte)?;
+        }
+        Ok(())
+    }
+}
+
+#[cfg(feature = "arithmetic")]
+#[cfg_attr(docsrs, doc(cfg(feature = "arithmetic")))]
+impl<C> str::FromStr for Signature<C>
+where
+    C: PrimeCurve + ScalarArithmetic,
+    SignatureSize<C>: ArrayLength<u8>,
+{
+    type Err = Error;
+
+    fn from_str(hex: &str) -> Result<Self> {
+        if hex.as_bytes().len() != C::UInt::BYTE_SIZE * 4 {
+            return Err(Error::new());
+        }
+
+        if !hex
+            .as_bytes()
+            .iter()
+            .all(|&byte| matches!(byte, b'0'..=b'9' | b'a'..=b'z' | b'A'..=b'Z'))
+        {
+            return Err(Error::new());
+        }
+
+        let (r_hex, s_hex) = hex.split_at(C::UInt::BYTE_SIZE * 2);
+
+        let r = r_hex
+            .parse::<NonZeroScalar<C>>()
+            .map_err(|_| Error::new())?;
+
+        let s = s_hex
+            .parse::<NonZeroScalar<C>>()
+            .map_err(|_| Error::new())?;
+
+        Self::from_scalars(r, s)
+    }
+}
+
+#[cfg(feature = "serde")]
+#[cfg_attr(docsrs, doc(cfg(feature = "serde")))]
+impl<C> Serialize for Signature<C>
+where
+    C: PrimeCurve,
+    SignatureSize<C>: ArrayLength<u8>,
+{
+    #[cfg(not(feature = "alloc"))]
+    fn serialize<S>(&self, serializer: S) -> core::result::Result<S::Ok, S::Error>
+    where
+        S: ser::Serializer,
+    {
+        self.as_ref().serialize(serializer)
+    }
+
+    #[cfg(feature = "alloc")]
+    fn serialize<S>(&self, serializer: S) -> core::result::Result<S::Ok, S::Error>
+    where
+        S: ser::Serializer,
+    {
+        use alloc::string::ToString;
+        if serializer.is_human_readable() {
+            self.to_string().serialize(serializer)
+        } else {
+            self.as_ref().serialize(serializer)
+        }
+    }
+}
+
+// TODO(tarcieri): support deserialization with the `arithmetic` feature disabled
+#[cfg(all(feature = "arithmetic", feature = "serde"))]
+#[cfg_attr(docsrs, doc(cfg(all(feature = "arithmetic", feature = "serde"))))]
+impl<'de, C> Deserialize<'de> for Signature<C>
+where
+    C: PrimeCurve + ScalarArithmetic,
+    SignatureSize<C>: ArrayLength<u8>,
+{
+    #[cfg(not(feature = "alloc"))]
+    fn deserialize<D>(deserializer: D) -> core::result::Result<Self, D::Error>
+    where
+        D: de::Deserializer<'de>,
+    {
+        use de::Error;
+        <&[u8]>::deserialize(deserializer)
+            .and_then(|bytes| Self::try_from(bytes).map_err(D::Error::custom))
+    }
+
+    #[cfg(feature = "alloc")]
+    fn deserialize<D>(deserializer: D) -> core::result::Result<Self, D::Error>
+    where
+        D: de::Deserializer<'de>,
+    {
+        use de::Error;
+        if deserializer.is_human_readable() {
+            <&str>::deserialize(deserializer)?
+                .parse()
+                .map_err(D::Error::custom)
+        } else {
+            <&[u8]>::deserialize(deserializer)
+                .and_then(|bytes| Self::try_from(bytes).map_err(D::Error::custom))
+        }
+    }
 }
